package ch.sws.tasaf.bo;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import org.apache.log4j.Logger;
import ch.sws.tasaf.bo.vo.*;
import ch.sws.tasaf.db.BookingDAO;
import ch.sws.tasaf.db.CountryDAO;
import ch.sws.tasaf.db.TransportDAO;

/**
 * Die Klasse TravelAssembler ist fr die Logic der Erstellung der Value-Objects fr
 * die Reisebuchung verantwortlich.
 * <p>
 * Copyright (c) 2004, Eidgenssisches Institut fr Geistiges Eigentum
 * @author    Anita Rueegsegger, Marc Bouquet
 * @version   1.0
 */
public class TravelAssembler
{
  // Logging
  private static final Logger LOG = Logger.getLogger( TravelAssembler.class );

  // DataAccessObjects
  private CountryDAO countryDAO = null;
  private TransportDAO transportDAO = null;

  // Value-Objects
  private BookingVO bookingVO = null;
  private ArrayList participants = null;
  private ArrayList programs = null;
  private ArrayList transports = null;

  /**
   * Standard Konstruktor
   * @param bookingVO BookingVO
   */
  public TravelAssembler( )
  {
    this.bookingVO = new BookingVO();
  }

  /**
   * Gibt ein Booking-Value-Objekt zurck
   * @throws Exception Unerwarteter Fehler
   * @return BookingVO
   */
  public BookingVO getBookingVO()
  {
    return bookingVO;
  }

  /**
   * bergibt ein Booking-Value-Objekt
   * @param bookingVO BookingVO
   * @throws Exception Unerwarteter Fehler
   */
  public String saveBooking( BookingVO bookingVO ) throws Exception
  {
    BookingDAO bookingDAO = new BookingDAO();
    bookingDAO.insert( bookingVO );

    return "Buchung erfolgreich";
  }

  /**
   * Gibt eine ArrayListe der Teilnehmer zurck, nachdem ein weiterer
   * Teilnehmer hinzugefgt wurde
   * @param participantVO ParticipantVO
   * @return ArrayList
   */
  public ArrayList addTN( ParticipantVO participantVO )
  {
    if ( participants == null )
    {
      participants = new ArrayList();
    }
    LOG.debug( "addTN: " + participantVO.toString() );
    participants.add( participantVO );
    bookingVO.setParticipants( participants );
    // Setzt die Anzahl Teilnehmer im BookingVO
    bookingVO.setAnzahlTN( participants.size() );
    return participants;
  }

  /**
   * Gibt eine aktualisierte ArrayListe der Teilnehmer zurck,
   * nachdem ein Teilnehmer gelscht wurde
   * @param index int
   * @throws Exception
   * @return ArrayList
   */
  public ArrayList removeTN( int index ) throws Exception
  {
    if ( participants == null )
    {
      LOG.warn( "Es kann kein Teilnehmer gelscht werden, da keiner Vorhanden ist!" );
      throw new Exception( "Es kann kein Teilnehmer gelscht werden, da keiner Vorhanden ist!" );
    }
    participants.remove( index );
    LOG.debug( "participantVO der Position: " + index + " wird aus der ArrayListe gelscht." );

    if ( participants.size() == 0 )
    {
      participants = null;
      bookingVO.setParticipants( participants );
      return participants;
    }
    else if ( participants.size() < 0 )
    {
      LOG.warn( "Es hat keine Reiseteilnehmer vorhanden." );
      throw new Exception( "Es hat keine Reiseteilnehmer vorhanden." );
    }
    else
    {
      participants = sortParticipants( participants );
      bookingVO.setParticipants( participants );
      // Setzt die Anzahl Teilnehmer im BookingVO
      bookingVO.setAnzahlTN( participants.size() );
      return participants;
    }
  }

  /**
   * Sortiert die Participants-ArrayListe neu mit den aktuellen Indexen
   * @param participants ArrayList
   * @return ArrayList sortList
   */
  private ArrayList sortParticipants( ArrayList participants )
  {
    ArrayList sortList = new ArrayList();
    int index = 0;
    Iterator iter = participants.iterator();
    while ( iter.hasNext() )
    {
      ParticipantVO participantVO = ( ParticipantVO ) iter.next();
      participantVO.setIndex( index );
      sortList.add( participantVO );
      index++;
    }
    return sortList;
  }

  /**
   * Gibt eine ArrayListe der Programme zurck, nachdem ein weiteres
   * Programm hinzugefgt wurde
   * @param programVO ProgramVO
   * @return ArrayList
   */
  public ArrayList addProgram( ProgramVO programVO )
  {
    if ( programs == null )
    {
      programs = new ArrayList();
    }
    LOG.debug( "addProgram: " + programVO.toString() );
    programs.add( programVO );
    bookingVO.setPrograms( programs );
    return programs;
  }

  /**
   * Gibt eine aktualisierte ArrayListe der Programme zurck,
   * nachdem ein Programm gelscht wurde
   * @param index int
   * @throws Exception
   * @return ArrayList
   */
  public ArrayList removeProgram( int index ) throws Exception
  {
    if ( programs == null )
    {
      LOG.warn( "Es knnen keine Program-Daten gelscht werden, da keine Vorhanden ist!" );
      throw new Exception( "Es knnen keine Program-Daten gelscht werden, da keine Vorhanden ist!" );
    }
    programs.remove( index );
    LOG.debug( "programVO der Position: " + index + " wird aus der ArrayListe gelscht." );

    if ( programs.size() == 0 )
    {
      programs = null;
      bookingVO.setParticipants( programs );
      return participants;
    }
    else if ( programs.size() < 0 )
    {
      LOG.warn( "Es sind keine Programm-Daten vorhanden." );
      throw new Exception( "Es sind keine Programm-Daten vorhanden." );
    }
    else
    {
      programs = sortPrograms( programs );
      bookingVO.setPrograms( programs );
      return programs;
    }
  }

  /**
   * Sortiert die Programs-ArrayListe neu mit den aktuellen Indexen
   * @param programs ArrayList
   * @return ArrayList sortList
   */
  public ArrayList sortPrograms( ArrayList programs )
  {
    ArrayList sortList = new ArrayList();
    int index = 0;
    Iterator iter = programs.iterator();
    while ( iter.hasNext() )
    {
      ProgramVO programVO = ( ProgramVO ) iter.next();
      programVO.setIndex( index );
      sortList.add( programVO );
      index++;
    }
    return sortList;
  }

  /**
   * Gibt eine aktualisierte ArrayListe zurck, nachdem neue
   * Daten eingefgt wurden
   * @param programsUpdate ArrayList
   * @return ArrayList
   */
  public ArrayList updatePrograms( ArrayList programsUpdate )
  {
    if ( programs == null )
    {
      programs = new ArrayList();
    }
    programs = programsUpdate;
    LOG.debug( "BookingVO (updatePrograms):\n" + bookingVO.toString() );
    bookingVO.setPrograms( programs );
    return programs;
  }

  /**
   * Fgt das Travel-Value-Objekt in das Booking-Value-Objekt ein
   * @param travelVO TravelVO
   */
  public void addTravel( TravelVO travelVO )
  {
    bookingVO.setTravelVO( travelVO );
    LOG.debug( travelVO.toString() );
  }

  /**
   * Fgt das Person-Value-Objekt in das Booking-Value-Objekt ein
   * @param personVO PersonVO
   */
  public void addPerson( PersonVO personVO )
  {
    bookingVO.setPersonVO( personVO );
    LOG.debug( personVO.toString() );
  }

  /**
   * Fgt die ausgewhlten Transportarten in das Booking-Value-Objekt ein
   * @param transports ArrayList TravelTransportVO
   * @throws Exception Unerwarteter Fehler
   */
  public void addTransports( ArrayList transports ) throws Exception
  {
    if ( transports == null )
    {
      transports = new ArrayList();
    }
    LOG.debug( "addTransports: " + transports.toString() );
    this.transports = transports;
    bookingVO.setTransports( this.transports );
  }

  /**
   * Gibt eine ArrayListe mit den Countries zurck
   * @throws Exception Unerwarteter Fehler
   * @return ArrayList CountryVO
   */
  public ArrayList getCountry() throws Exception
  {
    if ( countryDAO == null )
    {
      countryDAO = new CountryDAO();
    }
    try
    {
      return countryDAO.select();
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "SQL-Fehler beim Laden der CountryDaten: " + sqlex.getMessage() );
      throw new Exception( "SQL-Fehler beim Laden der CountryDaten: " + sqlex.getMessage() );
    }
  }

  /**
   * Gibt eine ArrayListe mit den TransportArten zurck
   * @throws Exception Unerwarteter Fehler
   * @return ArrayList TransportVO
   */
  public ArrayList getTransportArten() throws Exception
  {
    if ( transportDAO == null )
    {
      transportDAO = new TransportDAO();
    }
    try
    {
      return transportDAO.select();
    }
    catch ( SQLException sqlex )
    {
      LOG.error( "SQL-Fehler beim Laden der TransportArten: " + sqlex.getMessage() );
      throw new Exception( "SQL-Fehler beim Laden der TransportArten: " + sqlex.getMessage() );
    }
  }
}
