package ch.sws.tasaf.db;

import java.sql.*;
import java.util.ArrayList;
import java.util.Iterator;
import org.apache.log4j.Logger;
import ch.sws.tasaf.bo.vo.*;

/**
 * Kapselung der DB-Zugriffe fr die inserts in die Datenbank
 * <p>
 * Copyright (c) 2004, Eidgenssisches Institut fr Geistiges Eigentum
 * @author Anita Rueegsegger, Marc Bouquet
 * @version 1.0
 */
public class BookingDAO
{
  private static final Logger LOG = Logger.getLogger( BookingDAO.class );

  /**
   * Standard Konstruktor
   * @throws Exception
   */
  public BookingDAO() throws Exception
  {
  }

  /**
   * Fgt alle Daten vom BookingVO in die entsprechenden Tabellen ein
   * -  PERSON
   * -  TRAVEL
   * -  PARTICIPANT
   * -  PROGRAM
   * -  TRAVELTRANSPORT
   * Alle inserts werden innerhalb einer Transaktion durchgefhrt
   * Connection wird aus Tomcat-Connection-Pool geholt
   * @param bookingVO BookingVO
   * @throws Exception
   */
  public void insert( BookingVO bookingVO ) throws Exception
  {
    Connection con = null;
    try
    {
      con = DBHelper.getInstance().getConnection( DBHelper.MYSQL, DBHelper.JDBC_MYSQL );
      con.setAutoCommit( false );

      int idPerson = insertPerson( con, bookingVO.getPersonVO() );
      int idTravel = insertTravel( con, bookingVO.getTravelVO(), idPerson );
      this.insertParticipant( con, bookingVO.getParticipants(), idTravel );
      this.insertProgram( con, bookingVO.getPrograms(), idTravel );
      this.insertTravelTransport( con, bookingVO.getTransports(), idTravel );

      con.commit();
    }
    catch ( SQLException sqlex )
    {
      LOG.debug( "BookinkDAO.insert(): " + sqlex.getMessage() );
      con.rollback();
      throw new SQLException( "BookinkDAO.insert(): " + sqlex.getMessage() );
    }
    finally
    {
      if ( con != null )
      {
        con.close();
      }
    }
  }

  /**
   * Insert in die Tabelle PERSON
   * @param con Connection
   * @param personVO PersonVO
   * @throws SQLException
   * @return int idPerson des zuletzt eingefgten Datensatzes
   */
  private int insertPerson( Connection con, PersonVO personVO ) throws SQLException
  {
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    int idPerson = 0;

    try
    {
      query = "INSERT INTO person (countryCd, lastname, firstname, address, zipCode, city, " +
              "email, creditcartType, creditcartNumber, creditcartName, creditcartExpiryDate) " +
              "VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )";

      ps = con.prepareStatement( query );
      ps.setString( 1, personVO.getCountryCd() );
      ps.setString( 2, personVO.getLastname() );
      ps.setString( 3, personVO.getFirstname() );
      ps.setString( 4, personVO.getAddress() );
      ps.setInt( 5, personVO.getZipCode() );
      ps.setString( 6, personVO.getCity() );
      ps.setString( 7, personVO.getEmail() );
      ps.setString( 8, personVO.getCcType() );
      ps.setString( 9, personVO.getCcNumber() );
      ps.setString( 10, personVO.getCcName() );
      ps.setDate( 11, personVO.getCcExpiryDate() );

      ps.executeUpdate();

      query = "SELECT MAX(idPerson) idPerson FROM person";
      ps = con.prepareStatement( query );
      rs = ps.executeQuery();

      if ( rs.next() )
      {
        idPerson = rs.getInt( "idPerson" );
      }
      else
      {
        LOG.debug( "Personen-Key konnte nicht selektiert werden" );
        throw new SQLException( "Personen-Key konnte nicht selektiert werden" );
      }

      return idPerson;
    }
    catch ( SQLException sqlex )
    {
      LOG.debug( "BookinkDAO.insertPerson(): " + sqlex.getMessage() );
      throw new SQLException( "BookinkDAO.insertPerson(): " + sqlex.getMessage() );
    }
    finally
    {
      if ( ps != null )
      {
        ps.close();
      }
    }
  }

  /**
   * Insert in die Tabelle TRAVEL
   * @param con Connection
   * @param travelVO TravelVO
   * @param idPerson int Foreign-Key der Tabelle PERSON
   * @throws SQLException
   * @return int idTravel des zuletzt eingefgten Datensatzes
   */
  private int insertTravel( Connection con, TravelVO travelVO, int idPerson ) throws SQLException
  {
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    int idTravel = 0;

    try
    {
      query = "INSERT INTO travel(idPerson, startCity, startDate, endDate, cost) " +
              "VALUES ( ?, ?, ?, ?, ? )";

      ps = con.prepareStatement( query );
      ps.setInt( 1, idPerson );
      ps.setString( 2, travelVO.getStartCity() );
      ps.setDate( 3, travelVO.getStartDate() );
      ps.setDate( 4, travelVO.getEndDate() );
      ps.setFloat( 5, travelVO.getCosts() );
      ps.executeUpdate();

      query = "SELECT MAX(idTravel) idTravel FROM travel";
      ps = con.prepareStatement( query );
      rs = ps.executeQuery();

      if ( rs.next() )
      {
        idTravel = rs.getInt( "idTravel" );
      }
      else
      {
        LOG.debug( "Travel-Key konnte nicht selektiert werden" );
        throw new SQLException( "Travel-Key konnte nicht selektiert werden" );
      }

      return idTravel;
    }
    catch ( SQLException sqlex )
    {
      LOG.debug( "BookinkDAO.insertTravel(): " + sqlex.getMessage() );
      throw new SQLException( "BookinkDAO.insertTravel(): " + sqlex.getMessage() );
    }
    finally
    {
      if ( ps != null )
      {
        ps.close();
      }
    }
  }

  /**
   * Inserts die Tabelle PARTICIPANT
   * @param con Connection
   * @param participants ArrayList
   * @param idTravel int Foreign-Key der Tabelle TRAVEL
   * @throws SQLException
   */
  private void insertParticipant( Connection con, ArrayList participants, int idTravel ) throws SQLException
  {
    PreparedStatement ps = null;
    String query = null;

    try
    {
      query = "INSERT INTO participant (idTravel, countryCd, firstname, lastname, city) " +
              "VALUES ( ?, ?, ?, ?, ? )";

      Iterator iter = participants.iterator();
      while ( iter != null && iter.hasNext() )
      {
        ParticipantVO participantVO = ( ParticipantVO ) iter.next();
        ps = con.prepareStatement( query );
        ps.setInt( 1, idTravel );
        ps.setString( 2, participantVO.getCountryCd() );
        ps.setString( 3, participantVO.getFirstname() );
        ps.setString( 4, participantVO.getLastname() );
        ps.setString( 5, participantVO.getCity() );
        ps.executeUpdate();
      }
    }
    catch ( SQLException sqlex )
    {
      LOG.debug( "BookinkDAO.insertParticipant(): " + sqlex.getMessage() );
      throw new SQLException( "BookinkDAO.insertParticipant(): " + sqlex.getMessage() );
    }
    finally
    {
      if ( ps != null )
      {
        ps.close();
      }
    }
  }

  /**
   * Insert in die Tabelle PROGRAM
   * @param con Connection
   * @param programs ArrayList
   * @param idTravel int Foreign-Key der Tabelle TRAVEL
   * @throws SQLException
   */
  private void insertProgram( Connection con, ArrayList programs, int idTravel ) throws SQLException
  {
    PreparedStatement ps = null;
    String query = null;

    try
    {
      query = "INSERT INTO program (idHotel, idTravel, fromDate, toDate, numberOfRoubleRoom, numberOfSingleRoom) " +
              "VALUES ( ?, ?, ?, ?, ?, ? )";

      Iterator iter = programs.iterator();
      while ( iter != null && iter.hasNext() )
      {
        ProgramVO programVO = ( ProgramVO ) iter.next();

        ps = con.prepareStatement( query );
        ps.setInt( 1, programVO.getHotelVO().getIdHotel() );
        ps.setInt( 2, idTravel );
        ps.setDate( 3, ( java.sql.Date ) programVO.getFromDate() );
        ps.setDate( 4, ( java.sql.Date ) programVO.getToDate() );
        ps.setInt( 5, programVO.getNumberOfDoubleRoom() );
        ps.setInt( 6, programVO.getNumberOfSingleRoom() );
        ps.executeUpdate();
      }
    }
    catch ( SQLException sqlex )
    {
      LOG.debug( "BookinkDAO.insertProgram(): " + sqlex.getMessage() );
      throw new SQLException( "BookinkDAO.insertProgram(): " + sqlex.getMessage() );
    }
    finally
    {
      if ( ps != null )
      {
        ps.close();
      }
    }
  }

  /**
   * Insert in die Tabelle TRAVELTRANSPORT
   * @param con Connection
   * @param transports ArrayList
   * @param idTravel int Foreign-Key der Tabelle TRAVEL
   * @throws SQLException
   */
  private void insertTravelTransport( Connection con, ArrayList transports, int idTravel ) throws SQLException
  {
    PreparedStatement ps = null;
    String query = null;

    try
    {
      query = "INSERT INTO traveltransport (idTravel, idTransport) " +
              "VALUES ( ?, ? )";

      Iterator iter = transports.iterator();

      while ( iter != null && iter.hasNext() )
      {
        TravelTransportVO travelTransportVO = ( TravelTransportVO ) iter.next();

        ps = con.prepareStatement( query );
        ps.setInt( 1, idTravel );
        ps.setInt( 2, travelTransportVO.getTransportId() );
        ps.executeUpdate();
      }
    }
    catch ( SQLException sqlex )
    {
      LOG.debug( "BookingDAO.insertTravelTransport(): " + sqlex.getMessage() );
      throw new SQLException( "BookingDAO.insertTravelTransport(): " + sqlex.getMessage() );
    }
    finally
    {
      if ( ps != null )
      {
        ps.close();
      }
    }
  }
}
