package ch.sws.tasaf.db;

import java.sql.*;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import org.apache.log4j.Logger;

/**
 * Die Klasse DBHelper ist eine Hilfsklasse, welche die Connection auf die Datenbank erstellt
 * und dies wieder schliesst.
 * <p>
 * Copyright (c) 2004, Eidgenssisches Institut fr Geistiges Eigentum
 * @author Anita Rueegsegger, Marc Bouquet
 * @version 1.0
 */
public class DBHelper
{
  private static DBHelper dbHelper = null; // stat. Feld fr Singleton

  public static final String MYSQL = "jdbc:mysql://localhost:3306/tasaf";
  public static final String JDBC_MYSQL = "com.mysql.jdbc.Driver";
  private static final Logger LOG = Logger.getLogger( DBHelper.class );

  private String dbUser = "admin";
  private String dbPwd = "admin";

  /** Konstruktor - Singleton. */
  private DBHelper()
  {}

  /**
   * statische Methode, die eine Referenz auf den DBHelper
   * zurckgibt. Hier ist das Singleton-Pattern implementiert.
   * @return Referenz auf DBHelper (= me)
   */
  public static DBHelper getInstance()
  {
    if ( dbHelper == null )
    {
      dbHelper = new DBHelper();
    }
    return dbHelper;
  }

  /**
   * Mit der Methode getConnection wird die Verbindung auf die Datenbank erstellt
   * @param dbName Name der Datenbank
   * @param jdbcDriver Driver fr die JDBC-Schnittstelle
   * @return Connection
   */
  public Connection getConnection( String dbName, String jdbcDriver )
  {
    Connection con = null;
    try
    {
      // Driver registieren
      Class.forName( jdbcDriver );

      // Connect zur Datenbank ausfhren
      con = DriverManager.getConnection( dbName, dbUser, dbPwd );
      con.setAutoCommit( true );

      return con;
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception: " + ex.getMessage() );
      ex.printStackTrace();
      return null;
    }
  }

  /**
   * Schliesst die Connection und das Statement
   * @param con Connection
   * @param stmt Statement
   * @throws SQLException Unerwarterer SQL-Fehler
   */
  public void close( Connection con, Statement stmt ) throws SQLException
  {
    try
    {
      if ( con != null )
      {
        con.close();
      }
      if ( stmt != null )
      {
        stmt.close();
      }
    }
    catch ( SQLException sqlex )
    {
      LOG.warn( "SQLException: " + sqlex.getMessage() );
      throw new SQLException( "Connection kann nicht geschlossen werden:" + sqlex.getMessage() );
    }
  }

  /**
   * Schliesst die Connection und das PreparedStatement
   * @param con Connection
   * @param ps PreparedStatement
   * @throws SQLException Unerwarteter SQL-Fehler
   */
  public void close( Connection con, PreparedStatement ps ) throws SQLException
  {
    try
    {
      if ( con != null )
      {
        con.close();
      }
      if ( ps != null )
      {
        ps.close();
      }
    }
    catch ( SQLException sqlex )
    {
      LOG.warn( "SQLException: " + sqlex.getMessage() );
      throw new SQLException( "Connection kann nicht geschlossen werden:" + sqlex.getMessage() );
    }
  }

  /**
   * Gibt einen String in korrekter SQL-Syntax fr dynamisches SQL zurck. D.h. jedes "'" verdoppelt und am Anfang
   * und Ende ein "'"
   * @param str String der umgewandelt werden soll
   * @return String umgewandelt in SQL-Syntax fr dynamisches SQL
   */
  public String getSqlString( String str )
  {
    StringBuffer sb = new StringBuffer( "'" );

    try
    {
      StringTokenizer st = new StringTokenizer( str, "'", true );

      while ( st.hasMoreTokens() )
      {
        String wort = st.nextToken();
        sb.append( wort );

        if ( wort.equals( "'" ) )
        {
          sb.append( "'" );
        }
      }
    }
    catch ( NoSuchElementException nsex )
    {
      LOG.warn( "NoSuchElementException: " + nsex.getMessage() );
      nsex.printStackTrace();
    }
    sb.append( "'" );
    return sb.toString();
  }
}
