package ch.sws.tasaf.util;

import java.io.File;

/**
 * Hilfsklasse zur Initialisierung des Logging (Log4j)
 * <p>
 * Copyright (c) 2004, Eidgenssisches Institut fr Geistiges Eigentum
 * @author Anita Rueegsegger, Marc Bouquet
 * @version 1.0
 */
public class ClientLogInit
{
  /** the global default logging configuration file */
  final public static String DEFAULT_CONFIG = "/ch/sws/tasaf/util/default_config.xml";

  /** Default Verzeichnis fr die Logging-Daten */
  final public static String DEFAULT_DIRECTORY = "c:/data/log/";

  /**
   * initializes logging with the global default configuration file.
   * @see ClientLogInitalizer#initalizeLogging(String)
   */
  public static void initLogging()
  {
    ClientLogInit.checkLogDirectory( DEFAULT_DIRECTORY );
    ClientLogInit.initLogging( DEFAULT_CONFIG );
  }

  /**
   * berprft ob das Verzeichnis fr die Logging-Daten vorhanden ist, wenn nicht wird es erstellt.
   * @param path String path
   * @throws Exception Unerwarteter Fehler
   */
  private static void checkLogDirectory( String path )
  {
    try
    {
      File file = new File( path );
      if ( !file.isDirectory() )
      {
        file.mkdirs();
      }
    }
    catch ( Exception ex )
    {
      ClientLogInit.logMessage( "Verzeichnis (c:/data/log) konnte nicht erstellt werden." );
    }
  }

  /**
   * Log4J configures itself with the config file provided with the 'log4j.configuration' property.
   * check if the file which is specified with that property is really available.
   * otherwise a default config file is used.
   *
   * the 'log4j.configuration' property can be set in the JNLP launch file
   * or with a -Dlog4j.configuration=file:///c:/Programme/Ige/Log/config/jipps_config.xml parameter to the JVM.
   * NOTE: URL values like 'file://c:/xyz' are not supported by the java URL handler.
   *
   * @param defaultXMLConfigResource the XML config file in the classpath.
   */
  public static void initLogging( String defaultXMLConfigResource )
  {
    String message;
    try
    {
      //try to configure with the default config provided through the classloader
      final java.net.URL configFileURL = ( ClientLogInit.class ).getResource( defaultXMLConfigResource );
      org.apache.log4j.xml.DOMConfigurator.configure( configFileURL );
      message = "log4j default logging gestartet.";
    }
    catch ( Exception ex )
    {
      //if all fails we use the BasicConfigurator
      org.apache.log4j.BasicConfigurator.configure();
      message = "log4j basic logging gestartet.";
    }
    ClientLogInit.logMessage( message );
  }

  private static void logMessage( String message )
  {
    org.apache.log4j.Logger.getLogger( ClientLogInit.class ).debug( message );
  }
}
